
# q-expansion of newform 3120.2.g.n, downloaded from the LMFDB on 15 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3120
weight = 2
poly_data = [16, 0, 9, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[0, 5, 0, 1], 4], [[5, 0, 1, 0], 1]]

hecke_ring_character_values = [[1951, [1, 0, 0, 0]], [2341, [1, 0, 0, 0]], [2081, [1, 0, 0, 0]], [2497, [1, 0, 0, 0]], [2641, [-1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [-1, 0, 0, 0], [0, 0, 1, 0], [0, 1, -1, 0], [0, -1, 3, 0], [1, -1, 1, 1], [-2, 0, 0, -1], [0, -2, -4, 0], [-4, 0, 0, 1], [2, 0, 0, -4], [0, 0, -6, 0], [0, 1, -3, 0], [0, 1, 3, 0], [-4, 0, 0, 2], [0, 0, -4, 0], [-2, 0, 0, 5], [0, 0, 4, 0], [-10, 0, 0, -1], [0, 2, 8, 0], [0, -3, 1, 0], [0, 2, -6, 0], [4, 0, 0, -1], [0, -6, -2, 0], [0, -3, -9, 0], [0, -7, -3, 0], [-2, 0, 0, 0], [0, 0, 0, 0], [4, 0, 0, 3], [0, -2, -10, 0], [-14, 0, 0, 4], [-8, 0, 0, 6], [-4, 0, 0, -4], [0, -4, -10, 0], [-16, 0, 0, 1], [0, 7, 5, 0], [0, -4, -10, 0], [-10, 0, 0, 2], [0, 1, -1, 0], [0, -4, -8, 0], [-6, 0, 0, 4], [4, 0, 0, 4], [-6, 0, 0, 1], [-12, 0, 0, -4], [0, -7, -15, 0], [0, -4, 2, 0], [8, 0, 0, 0], [-4, 0, 0, 0], [0, 2, 8, 0], [0, 12, 4, 0], [0, -2, 14, 0], [18, 0, 0, -3], [0, 1, -11, 0], [0, -10, -6, 0], [-8, 0, 0, -6], [-2, 0, 0, 0], [20, 0, 0, -4], [2, 0, 0, 10], [0, -4, 18, 0], [6, 0, 0, 2], [0, 0, -22, 0], [-4, 0, 0, 14], [0, 4, 2, 0], [0, -7, -21, 0], [-12, 0, 0, 2], [14, 0, 0, -2], [0, 0, 6, 0], [0, -10, -16, 0], [-26, 0, 0, 0], [0, 0, 0, 7], [0, -2, -14, 0], [0, 2, 24, 0], [0, -4, 16, 0], [8, 0, 0, -2], [18, 0, 0, -10], [0, -4, -14, 0], [0, -6, -10, 0], [-6, 0, 0, -12], [0, 13, 9, 0], [0, 8, 14, 0], [0, 0, -12, 0], [-12, 0, 0, 10], [0, -6, 10, 0], [0, -12, 0, 0], [-2, 0, 0, 12], [-20, 0, 0, -5], [4, 0, 0, -1], [0, -1, 1, 0], [0, 1, 29, 0], [0, -11, -9, 0], [0, 5, -29, 0], [16, 0, 0, -7], [0, 9, 5, 0], [0, 9, 19, 0], [24, 0, 0, -8], [0, 6, 16, 0], [20, 0, 0, 8], [0, 7, 25, 0], [-34, 0, 0, 6], [-12, 0, 0, 10], [0, 8, 16, 0], [-20, 0, 0, -10], [0, 8, 14, 0], [-4, 0, 0, -11], [2, 0, 0, 8], [-8, 0, 0, -3], [0, -15, -23, 0], [0, -2, 2, 0], [0, -8, -34, 0], [0, 0, 0, 4], [-6, 0, 0, -7], [40, 0, 0, -4], [0, 5, 9, 0], [0, 4, -6, 0], [0, 0, -30, 0], [0, 16, 2, 0], [6, 0, 0, 2], [0, -1, 17, 0], [0, 0, 0, -17], [-2, 0, 0, -12], [24, 0, 0, -10], [0, 8, -12, 0], [-6, 0, 0, 18], [14, 0, 0, -15], [0, -2, 14, 0], [0, 0, 18, 0], [10, 0, 0, -6], [0, 0, 48, 0], [-4, 0, 0, 2], [-16, 0, 0, -6], [0, 11, 3, 0], [0, -12, 6, 0], [0, 16, 16, 0], [28, 0, 0, -13], [18, 0, 0, 2], [0, -4, -10, 0], [0, 0, 8, 0], [0, -8, 2, 0], [0, -2, -12, 0], [-14, 0, 0, 1], [-10, 0, 0, -2], [0, -8, -18, 0], [0, 9, 15, 0], [0, 0, 0, 10], [0, -12, 4, 0], [18, 0, 0, -12], [0, 3, 15, 0], [0, 7, -5, 0], [-6, 0, 0, 1], [0, 0, 0, 17], [0, -6, -2, 0], [0, 2, -22, 0], [-38, 0, 0, 6], [4, 0, 0, 8], [-12, 0, 0, -3], [-4, 0, 0, -12], [-12, 0, 0, 6], [12, 0, 0, -21], [0, 3, -27, 0], [6, 0, 0, 8], [0, 3, 17, 0], [0, 8, 36, 0], [26, 0, 0, 3], [0, 18, 8, 0], [-24, 0, 0, -2], [0, -6, 12, 0], [0, 12, -12, 0], [4, 0, 0, -23], [14, 0, 0, -18], [0, -2, -38, 0], [-18, 0, 0, 3], [0, 12, 12, 0], [0, -20, -8, 0], [12, 0, 0, -18], [0, -7, -3, 0], [-44, 0, 0, 3], [18, 0, 0, -10], [0, 2, -12, 0], [0, 9, 11, 0], [32, 0, 0, 8], [2, 0, 0, -4], [0, -3, 11, 0], [4, 0, 0, -18], [-22, 0, 0, 4], [0, -6, 36, 0], [0, -14, -10, 0], [10, 0, 0, 10], [-10, 0, 0, 6], [0, -15, -9, 0], [0, 18, -10, 0], [0, -3, -59, 0], [-22, 0, 0, 14], [0, 4, -8, 0], [-8, 0, 0, 27], [0, 3, 13, 0], [8, 0, 0, 0], [26, 0, 0, -9], [0, 18, -6, 0], [10, 0, 0, -16], [0, -16, 6, 0], [-20, 0, 0, 3], [0, -24, -14, 0], [4, 0, 0, 15], [0, 5, 61, 0], [30, 0, 0, -9], [0, -15, -31, 0], [-34, 0, 0, 15], [0, -20, -10, 0], [-16, 0, 0, -12], [0, -1, 13, 0], [-40, 0, 0, -5], [14, 0, 0, -20], [30, 0, 0, -22], [32, 0, 0, -6], [0, 6, -26, 0], [0, -8, -52, 0], [0, 6, -26, 0], [-24, 0, 0, 8], [-22, 0, 0, 14], [0, -14, 10, 0], [0, -16, 6, 0], [26, 0, 0, 5], [0, 24, 6, 0], [0, 12, -2, 0], [0, 10, 40, 0], [-56, 0, 0, 1], [-54, 0, 0, 9], [58, 0, 0, -9], [0, 0, 0, 20], [40, 0, 0, -14], [0, -13, 35, 0], [-10, 0, 0, 10], [-40, 0, 0, -7], [0, 4, 50, 0], [-26, 0, 0, 0], [-12, 0, 0, 6], [0, -6, 50, 0], [0, 0, -4, 0], [0, -18, -12, 0], [60, 0, 0, 0], [-20, 0, 0, 20], [0, 0, 44, 0], [-8, 0, 0, 19], [0, 0, 0, -6], [0, -12, 16, 0], [0, -8, 2, 0], [-20, 0, 0, 12], [0, 14, -40, 0], [0, -9, -9, 0], [0, 4, 18, 0], [-48, 0, 0, 4], [0, -2, -14, 0], [0, -20, -22, 0], [0, -16, 24, 0], [-6, 0, 0, 5], [-14, 0, 0, 7], [0, 13, 17, 0], [58, 0, 0, -8], [0, -21, -7, 0], [-10, 0, 0, 32], [0, -3, 29, 0], [64, 0, 0, -2], [64, 0, 0, -7], [0, -4, 52, 0], [-10, 0, 0, -2], [0, -12, -22, 0], [8, 0, 0, 23], [0, -13, -51, 0], [0, -5, 5, 0], [0, -13, -35, 0], [18, 0, 0, 0], [26, 0, 0, 0], [0, 17, -13, 0], [0, 11, -1, 0], [-40, 0, 0, 8], [-34, 0, 0, -10], [0, 5, 19, 0], [0, -6, -62, 0], [0, -20, 0, 0], [0, 0, 48, 0], [0, 0, 0, 10], [4, 0, 0, 3], [0, -10, -4, 0], [16, 0, 0, -16], [0, 28, 24, 0], [0, -2, -20, 0], [16, 0, 0, 6], [-2, 0, 0, -8], [0, -6, 36, 0], [0, -10, -24, 0], [82, 0, 0, -2], [-26, 0, 0, -8], [68, 0, 0, 0], [0, -12, -10, 0], [0, 0, 4, 0], [-18, 0, 0, -4], [2, 0, 0, 4], [40, 0, 0, -20], [-62, 0, 0, -3], [-8, 0, 0, 18], [0, 1, 19, 0], [-38, 0, 0, -8], [0, -6, -44, 0], [-52, 0, 0, 9]]
